package example.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import example.domain.MailAddressInfo;
import example.domain.UserInfo;
import example.service.dao.UserInfoDao;

/**
 * UserInfoDao̎.
 *
 */
public class UserInfoDaoImpl implements UserInfoDao {
    /** [UIDV[PX */
    private static int uidSeq = 1;
    /** [U */
    private static Map<Integer, UserInfo> users;

    /** [AhXIDV[PX */
    private static int mailSeq = 1;
    
    static {
        users = new LinkedHashMap<Integer, UserInfo>();
        UserInfoDaoImpl dao = new UserInfoDaoImpl();
        {
            // _~[[U
            UserInfo info = new UserInfo();
            info.setName("User 01");
            dao.addUser(info);
            MailAddressInfo mailInfo1 = new MailAddressInfo();
            mailInfo1.setMailAddress("mail11@hoge.fuga");
            dao.addMailAddress(info, mailInfo1);
            MailAddressInfo mailInfo2 = new MailAddressInfo();
            mailInfo2.setMailAddress("mail12@hoge.fuga");
            dao.addMailAddress(info, mailInfo2);
        }
        {
            // _~[[U
            UserInfo info = new UserInfo();
            info.setName("User 02");
            dao.addUser(info);
            MailAddressInfo mailInfo = new MailAddressInfo();
            mailInfo.setMailAddress("mail21@hoge.fuga");
            dao.addMailAddress(info, mailInfo);
        }
    }
    
    public List<UserInfo> getAllUsers() {
        List<UserInfo> ret = new ArrayList<UserInfo>();
        
        for (Entry<Integer, UserInfo> entry : users.entrySet()) {
            ret.add(deepCopyUserInfo(entry.getValue()));
        }
        return ret;
    }

    public UserInfo findUser(int id) {
        return deepCopyUserInfo(users.get(new Integer(id)));
    }
    
    public void addUser(UserInfo userInfo) {
        synchronized (UserInfoDaoImpl.class) {
            // ID̔
            userInfo.setId(++uidSeq);
            users.put(new Integer(uidSeq), userInfo);
        }
    }

    public int updateUser(UserInfo userInfo) {
        Integer id = new Integer(userInfo.getId());
        if (!users.containsKey(id)) {
            // [UȂ
            return 0;
        }
        users.remove(id);
        users.put(id, deepCopyUserInfo(userInfo));
        return 1;
    }

    public void addMailAddress(UserInfo userInfo, MailAddressInfo mailAddressInfo) {
        Integer id = new Integer(userInfo.getId());
        synchronized (UserInfoDaoImpl.class) {
            // ID̔
            mailAddressInfo.setId(++mailSeq);
            UserInfo persisted = users.get(id);
            persisted.getMailAddresses().add(deepCopyMailAddress(mailAddressInfo));
        }
    }

    public int removeMailAddress(UserInfo userInfo, int id) {
        UserInfo persisted = users.get(userInfo.getId());
        for (Iterator it = persisted.getMailAddresses().iterator(); it.hasNext();) {
            MailAddressInfo mailAddressInfo = (MailAddressInfo) it.next();
            if (mailAddressInfo.getId() ==  id) {
                it.remove();
                return 1;
            }
        }
        return 0;
    }

    public int updateAddress(MailAddressInfo mailAddressInfo) {
        for (Entry<Integer, UserInfo> entry : users.entrySet()) {
            for (MailAddressInfo persisted : entry.getValue().getMailAddresses()) {
                if (persisted.getId() == mailAddressInfo.getId()) {
                    // Ώۂɐݒ
                    persisted.setMailAddress(mailAddressInfo.getMailAddress());
                    return 1;
                }
            }
        }
        return 0;
    }

    private UserInfo deepCopyUserInfo(UserInfo src) {
        UserInfo dest = new UserInfo();
        dest.setId(src.getId());
        dest.setName(src.getName());
        List<MailAddressInfo> addresses = new ArrayList<MailAddressInfo>();
        for (MailAddressInfo srcAddress : src.getMailAddresses()) {
            MailAddressInfo destAddress = new MailAddressInfo();
            destAddress.setId(srcAddress.getId());
            destAddress.setMailAddress(srcAddress.getMailAddress());
            addresses.add(srcAddress);
        }
        dest.setMailAddresses(addresses);
        return dest;
    }

    private MailAddressInfo deepCopyMailAddress(MailAddressInfo src) {
        MailAddressInfo dest = new MailAddressInfo();
        dest.setId(src.getId());
        dest.setMailAddress(src.getMailAddress());
        return dest;
    }
}
